@@ -25,8 +25,37 @@
  *-----------------------------------------------------------------------------
  */
 
+#include "tclThreadInt.h"
 #include "threadSvCmd.h"
 #include "tclXkeylist.h"
+#include <stdarg.h>
+
+#ifdef STATIC_BUILD
+#if TCL_MAJOR_VERSION >= 9
+/*
+ * Static build, Tcl >= 9, compile-time decision to disable T_ROT calls.
+ */
+#undef Tcl_RegisterObjType
+#define Tcl_RegisterObjType(typePtr)    (typePtr)->setFromAnyProc = NULL
+#else
+/*
+ * Static build, Tcl <= 9   --> T_ROT is directly linked, no stubs
+ * Nothing needs to be done
+ */
+#endif
+#else /* !STATIC_BUILD */
+/*
+ * Dynamic build. Assume building with stubs (xx) and make a run-time
+ * decision regarding T_ROT.
+ * (Ad xx): Should be checked. Without stubs we have to go like static.
+ */
+#undef Tcl_RegisterObjType
+#define Tcl_RegisterObjType(typePtr) if (threadTclVersion<90) { \
+    ((void (*)(const Tcl_ObjType *))((&(tclStubsPtr->tcl_PkgProvideEx))[211]))(typePtr); \
+} else { \
+    (typePtr)->setFromAnyProc = NULL; \
+}
+#endif /* eof STATIC_BUILD */
 
 /*---------------------------------------------------------------------------*/
 /*---------------------------------------------------------------------------*/
@@ -67,7 +96,6 @@
 static const char *tclXWrongArgs = "wrong # args: ";
 
 static const Tcl_ObjType *listType;
-static const Tcl_ObjType *stringType;
 
 /*-----------------------------------------------------------------------------
  * TclX_IsNullObj --
@@ -85,21 +113,15 @@ static int
 TclX_IsNullObj (objPtr)
     Tcl_Obj *objPtr;
 {
-    int length;
-
     if (objPtr->typePtr == NULL) {
         return (objPtr->length == 0);
-    } else {
-        if (objPtr->typePtr == listType) {
-            Tcl_ListObjLength (NULL, objPtr, &length);
-            return (length == 0);
-        } else if (objPtr->typePtr == stringType) {
-            Tcl_GetStringFromObj (objPtr, &length);
-            return (length == 0);
-        }
+    } else if (objPtr->typePtr == listType) {
+        int length;
+        Tcl_ListObjLength(NULL, objPtr, &length);
+        return (length == 0);
     }
-    Tcl_GetStringFromObj (objPtr, &length);
-    return (length == 0);
+    (void)Tcl_GetString(objPtr);
+    return (objPtr->length == 0);
 }
 
 /*-----------------------------------------------------------------------------
@@ -157,11 +179,8 @@ TclX_WrongArgs (interp, commandNameObj, string)
     Tcl_Obj     *commandNameObj;
     char        *string;
 {
-    const char *commandName;
-    Tcl_Obj *resultPtr = Tcl_GetObjResult (interp);
-    int      commandLength;
-
-    commandName = Tcl_GetStringFromObj (commandNameObj, &commandLength);
+    const char *commandName = Tcl_GetString(commandNameObj);
+    Tcl_Obj *resultPtr = Tcl_GetObjResult(interp);
 
     Tcl_ResetResult(interp);
     Tcl_AppendStringsToObj (resultPtr,
@@ -249,7 +268,7 @@ ValidateKeyedList(keylIntObj_t *keylIntPtr);
 static int
 ValidateKey(Tcl_Interp *interp,
                          const char *key,
-                         int keyLen,
+                         size_t keyLen,
                          int isPath);
 
 static keylIntObj_t *
@@ -354,7 +373,7 @@ ValidateKeyedList (keylIntPtr)
         TclX_Assert (entryPtr->key != NULL);
         TclX_Assert (entryPtr->valuePtr->refCount >= 1);
         if (entryPtr->valuePtr->typePtr == &keyedListType) {
-            ValidateKeyedList (entryPtr->valuePtr->internalRep.otherValuePtr);
+            ValidateKeyedList (entryPtr->valuePtr->internalRep.twoPtrValue.ptr1);
         }
     }
 }
@@ -375,24 +394,24 @@ ValidateKeyedList (keylIntPtr)
  *-----------------------------------------------------------------------------
  */
 static int
-ValidateKey (interp, key, keyLen, isPath)
+ValidateKey(interp, key, keyLen, isPath)
     Tcl_Interp *interp;
     const char *key;
-    int keyLen;
+    size_t keyLen;
     int isPath;
 {
     const char *keyp;
 
-    if (strlen (key) != (size_t) keyLen) {
+    if (strlen(key) != keyLen) {
         Tcl_ResetResult(interp);
-        Tcl_AppendStringsToObj (Tcl_GetObjResult (interp),
+        Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
                                 "keyed list key may not be a ",
                                 "binary string", (char *) NULL);
         return TCL_ERROR;
     }
-    if (key [0] == '\0') {
+    if (key[0] == '\0') {
         Tcl_ResetResult(interp);
-        Tcl_AppendStringsToObj (Tcl_GetObjResult (interp),
+        Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
                                 "keyed list key may not be an ",
                                 "empty string", (char *) NULL);
         return TCL_ERROR;
@@ -400,7 +419,7 @@ ValidateKey (interp, key, keyLen, isPath)
     for (keyp = key; *keyp != '\0'; keyp++) {
         if ((!isPath) && (*keyp == '.')) {
             Tcl_ResetResult(interp);
-            Tcl_AppendStringsToObj (Tcl_GetObjResult (interp),
+            Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
                                     "keyed list key may not contain a \".\"; ",
                                     "it is used as a separator in key paths",
                                     (char *) NULL);
@@ -596,35 +615,34 @@ ObjToKeyedListEntry (interp, objPtr, entryPtr)
     int objc;
     Tcl_Obj **objv;
     const char *key;
-    int keyLen;
 
-    if (Tcl_ListObjGetElements (interp, objPtr, &objc, &objv) != TCL_OK) {
+    if (Tcl_ListObjGetElements(interp, objPtr, &objc, &objv) != TCL_OK) {
         Tcl_ResetResult (interp);
-        Tcl_AppendStringsToObj (Tcl_GetObjResult (interp),
+        Tcl_AppendStringsToObj(Tcl_GetObjResult (interp),
                                 "keyed list entry not a valid list, ",
                                 "found \"",
-                                Tcl_GetStringFromObj (objPtr, NULL),
+                                Tcl_GetString(objPtr),
                                 "\"", (char *) NULL);
         return TCL_ERROR;
     }
 
     if (objc != 2) {
-        Tcl_AppendStringsToObj (Tcl_GetObjResult (interp),
+        Tcl_AppendStringsToObj(Tcl_GetObjResult (interp),
                                 "keyed list entry must be a two ",
                                 "element list, found \"",
-                                Tcl_GetStringFromObj (objPtr, NULL),
+                                Tcl_GetString(objPtr),
                                 "\"", (char *) NULL);
         return TCL_ERROR;
     }
 
-    key = Tcl_GetStringFromObj (objv [0], &keyLen);
-    if (ValidateKey (interp, key, keyLen, FALSE) == TCL_ERROR) {
+    key = Tcl_GetString(objv[0]);
+    if (ValidateKey(interp, key, objv[0]->length, FALSE) == TCL_ERROR) {
         return TCL_ERROR;
     }
 
-    entryPtr->key = ckstrdup (key);
-    entryPtr->valuePtr = Tcl_DuplicateObj (objv [1]);
-    Tcl_IncrRefCount (entryPtr->valuePtr);
+    entryPtr->key = ckstrdup(key);
+    entryPtr->valuePtr = Tcl_DuplicateObj(objv [1]);
+    Tcl_IncrRefCount(entryPtr->valuePtr);
 
     return TCL_OK;
 }
@@ -641,7 +659,7 @@ static void
 FreeKeyedListInternalRep (keylPtr)
     Tcl_Obj *keylPtr;
 {
-    FreeKeyedListData ((keylIntObj_t *) keylPtr->internalRep.otherValuePtr);
+    FreeKeyedListData(keylPtr->internalRep.twoPtrValue.ptr1);
 }
 
 /*-----------------------------------------------------------------------------
@@ -659,7 +677,7 @@ DupKeyedListInternalRep (srcPtr, copyPtr)
     Tcl_Obj *copyPtr;
 {
     keylIntObj_t *srcIntPtr =
-        (keylIntObj_t *) srcPtr->internalRep.otherValuePtr;
+        srcPtr->internalRep.twoPtrValue.ptr1;
     keylIntObj_t *copyIntPtr;
     int idx;
 
@@ -678,7 +696,7 @@ DupKeyedListInternalRep (srcPtr, copyPtr)
         Tcl_IncrRefCount (copyIntPtr->entries [idx].valuePtr);
     }
 
-    copyPtr->internalRep.otherValuePtr = (void *) copyIntPtr;
+    copyPtr->internalRep.twoPtrValue.ptr1 = copyIntPtr;
     copyPtr->typePtr = &keyedListType;
 
     KEYL_REP_ASSERT (copyIntPtr);
@@ -701,7 +719,7 @@ DupKeyedListInternalRepShared (srcPtr, copyPtr)
     Tcl_Obj *copyPtr;
 {
     keylIntObj_t *srcIntPtr =
-        (keylIntObj_t *) srcPtr->internalRep.otherValuePtr;
+        srcPtr->internalRep.twoPtrValue.ptr1;
     keylIntObj_t *copyIntPtr;
     int idx;
 
@@ -721,7 +739,7 @@ DupKeyedListInternalRepShared (srcPtr, copyPtr)
         Tcl_IncrRefCount(copyIntPtr->entries [idx].valuePtr);
     }
 
-    copyPtr->internalRep.otherValuePtr = (void *) copyIntPtr;
+    copyPtr->internalRep.twoPtrValue.ptr1 = copyIntPtr;
     copyPtr->typePtr = &keyedListType;
 
     KEYL_REP_ASSERT (copyIntPtr);
@@ -764,7 +782,7 @@ SetKeyedListFromAny (interp, objPtr)
         (objPtr->typePtr->freeIntRepProc != NULL)) {
         (*objPtr->typePtr->freeIntRepProc) (objPtr);
     }
-    objPtr->internalRep.otherValuePtr = (void *) keylIntPtr;
+    objPtr->internalRep.twoPtrValue.ptr1 = keylIntPtr;
     objPtr->typePtr = &keyedListType;
 
     KEYL_REP_ASSERT (keylIntPtr);
@@ -788,12 +806,12 @@ UpdateStringOfKeyedList (keylPtr)
     Tcl_Obj  *keylPtr;
 {
 #define UPDATE_STATIC_SIZE 32
-    int idx, strLen;
+    int idx;
     Tcl_Obj **listObjv, *entryObjv [2], *tmpListObj;
     Tcl_Obj *staticListObjv [UPDATE_STATIC_SIZE];
     char *listStr;
     keylIntObj_t *keylIntPtr =
-        (keylIntObj_t *) keylPtr->internalRep.otherValuePtr;
+        keylPtr->internalRep.twoPtrValue.ptr1;
 
     /*
      * Conversion to strings is done via list objects to support binary data.
@@ -812,16 +830,16 @@ UpdateStringOfKeyedList (keylPtr)
      */
     for (idx = 0; idx < keylIntPtr->numEntries; idx++) {
         entryObjv [0] =
-            Tcl_NewStringObj (keylIntPtr->entries [idx].key,
+            Tcl_NewStringObj(keylIntPtr->entries [idx].key,
                               strlen (keylIntPtr->entries [idx].key));
         entryObjv [1] = keylIntPtr->entries [idx].valuePtr;
         listObjv [idx] = Tcl_NewListObj (2, entryObjv);
     }
 
     tmpListObj = Tcl_NewListObj (keylIntPtr->numEntries, listObjv);
-    listStr = Tcl_GetStringFromObj (tmpListObj, &strLen);
-    keylPtr->bytes = ckbinstrdup (listStr, strLen);
-    keylPtr->length = strLen;
+    listStr = Tcl_GetString(tmpListObj);
+    keylPtr->bytes = ckbinstrdup(listStr, tmpListObj->length);
+    keylPtr->length = tmpListObj->length;
 
     Tcl_DecrRefCount (tmpListObj);
     if (listObjv != staticListObjv)
@@ -842,7 +860,7 @@ TclX_NewKeyedListObj ()
     Tcl_Obj *keylPtr = Tcl_NewObj ();
     keylIntObj_t *keylIntPtr = AllocKeyedListIntRep ();
 
-    keylPtr->internalRep.otherValuePtr = (void *) keylIntPtr;
+    keylPtr->internalRep.twoPtrValue.ptr1 = keylIntPtr;
     keylPtr->typePtr = &keyedListType;
     return keylPtr;
 }
@@ -875,9 +893,12 @@ TclX_KeyedListGet (interp, keylPtr, key, valuePtrPtr)
     const char *nextSubKey;
     int findIdx;
 
-    if (Tcl_ConvertToType (interp, keylPtr, &keyedListType) != TCL_OK)
-        return TCL_ERROR;
-    keylIntPtr = (keylIntObj_t *) keylPtr->internalRep.otherValuePtr;
+    if (keylPtr->typePtr != &keyedListType) {
+        if (SetKeyedListFromAny(interp, keylPtr) != TCL_OK) {
+            return TCL_ERROR;
+        }
+    }
+    keylIntPtr = keylPtr->internalRep.twoPtrValue.ptr1;
     KEYL_REP_ASSERT (keylIntPtr);
 
     findIdx = FindKeyedListEntry (keylIntPtr, key, NULL, &nextSubKey);
@@ -931,9 +952,12 @@ TclX_KeyedListSet (interp, keylPtr, key, valuePtr)
     int findIdx, keyLen, status;
     Tcl_Obj *newKeylPtr;
 
-    if (Tcl_ConvertToType (interp, keylPtr, &keyedListType) != TCL_OK)
-        return TCL_ERROR;
-    keylIntPtr = (keylIntObj_t *) keylPtr->internalRep.otherValuePtr;
+    if (keylPtr->typePtr != &keyedListType) {
+        if (SetKeyedListFromAny(interp, keylPtr) != TCL_OK) {
+            return TCL_ERROR;
+        }
+    }
+    keylIntPtr = keylPtr->internalRep.twoPtrValue.ptr1;
     KEYL_REP_ASSERT (keylIntPtr);
 
     findIdx = FindKeyedListEntry (keylIntPtr, key,
@@ -1028,9 +1052,12 @@ TclX_KeyedListDelete (interp, keylPtr, key)
     const char *nextSubKey;
     int findIdx, status;
 
-    if (Tcl_ConvertToType (interp, keylPtr, &keyedListType) != TCL_OK)
-        return TCL_ERROR;
-    keylIntPtr = (keylIntObj_t *) keylPtr->internalRep.otherValuePtr;
+    if (keylPtr->typePtr != &keyedListType) {
+        if (SetKeyedListFromAny(interp, keylPtr) != TCL_OK) {
+            return TCL_ERROR;
+        }
+    }
+    keylIntPtr = keylPtr->internalRep.twoPtrValue.ptr1;
 
     findIdx = FindKeyedListEntry (keylIntPtr, key, NULL, &nextSubKey);
 
@@ -1064,8 +1091,8 @@ TclX_KeyedListDelete (interp, keylPtr, key)
                                    keylIntPtr->entries [findIdx].valuePtr,
                                    nextSubKey);
     if (status == TCL_OK) {
-        subKeylIntPtr = (keylIntObj_t *)
-            keylIntPtr->entries [findIdx].valuePtr->internalRep.otherValuePtr;
+        subKeylIntPtr =
+            keylIntPtr->entries [findIdx].valuePtr->internalRep.twoPtrValue.ptr1;
         if (subKeylIntPtr->numEntries == 0) {
             DeleteKeyedListEntry (keylIntPtr, findIdx);
         }
@@ -1104,9 +1131,12 @@ TclX_KeyedListGetKeys (interp, keylPtr, key, listObjPtrPtr)
     const char *nextSubKey;
     int idx, findIdx;
 
-    if (Tcl_ConvertToType (interp, keylPtr, &keyedListType) != TCL_OK)
-        return TCL_ERROR;
-    keylIntPtr = (keylIntObj_t *) keylPtr->internalRep.otherValuePtr;
+    if (keylPtr->typePtr != &keyedListType) {
+        if (SetKeyedListFromAny(interp, keylPtr) != TCL_OK) {
+            return TCL_ERROR;
+        }
+    }
+    keylIntPtr = keylPtr->internalRep.twoPtrValue.ptr1;
 
     /*
      * If key is not NULL or empty, then recurse down until we go past
@@ -1158,22 +1188,20 @@ Tcl_KeylgetObjCmd (clientData, interp, objc, objv)
     Tcl_Obj     *const objv[];
 {
     Tcl_Obj *keylPtr, *valuePtr;
-    const char *varName, *key;
-    int keyLen, status;
+    const char *key;
+    int status;
 
     if ((objc < 2) || (objc > 4)) {
         return TclX_WrongArgs (interp, objv [0],
                                "listvar ?key? ?retvar | {}?");
     }
-    varName = Tcl_GetStringFromObj (objv [1], NULL);
-
     /*
      * Handle request for list of keys, use keylkeys command.
      */
     if (objc == 2)
         return Tcl_KeylkeysObjCmd (clientData, interp, objc, objv);
 
-    keylPtr = Tcl_GetVar2Ex(interp, varName, NULL, TCL_LEAVE_ERR_MSG);
+    keylPtr = Tcl_ObjGetVar2(interp, objv[1], NULL, TCL_LEAVE_ERR_MSG);
     if (keylPtr == NULL) {
         return TCL_ERROR;
     }
@@ -1181,8 +1209,8 @@ Tcl_KeylgetObjCmd (clientData, interp, objc, objv)
     /*
      * Handle retrieving a value for a specified key.
      */
-    key = Tcl_GetStringFromObj (objv [2], &keyLen);
-    if (ValidateKey (interp, key, keyLen, TRUE) == TCL_ERROR) {
+    key = Tcl_GetString(objv[2]);
+    if (ValidateKey(interp, key, objv[2]->length, TRUE) == TCL_ERROR) {
         return TCL_ERROR;
     }
 
@@ -1217,8 +1245,8 @@ Tcl_KeylgetObjCmd (clientData, interp, objc, objv)
     /*
      * Variable (or empty variable name) specified.
      */
-    if (!TclX_IsNullObj (objv [3])) {
-        if (Tcl_SetVar2Ex(interp, Tcl_GetStringFromObj(objv [3], NULL), NULL,
+    if (!TclX_IsNullObj(objv [3])) {
+        if (Tcl_ObjSetVar2(interp, objv[3], NULL,
                           valuePtr, TCL_LEAVE_ERR_MSG) == NULL)
             return TCL_ERROR;
     }
@@ -1241,21 +1269,20 @@ Tcl_KeylsetObjCmd (clientData, interp, objc, objv)
     Tcl_Obj     *const objv[];
 {
     Tcl_Obj *keylVarPtr, *newVarObj;
-    const char *varName, *key;
-    int idx, keyLen;
+    const char *key;
+    int idx;
 
     if ((objc < 4) || ((objc % 2) != 0)) {
         return TclX_WrongArgs (interp, objv [0],
                                "listvar key value ?key value...?");
     }
-    varName = Tcl_GetStringFromObj (objv [1], NULL);
 
     /*
      * Get the variable that we are going to update.  If the var doesn't exist,
      * create it.  If it is shared by more than being a variable, duplicated
      * it.
      */
-    keylVarPtr = Tcl_GetVar2Ex(interp, varName, NULL, 0);
+    keylVarPtr = Tcl_ObjGetVar2(interp, objv[1], NULL, 0);
     if ((keylVarPtr == NULL) || (Tcl_IsShared (keylVarPtr))) {
         if (keylVarPtr == NULL) {
             keylVarPtr = TclX_NewKeyedListObj ();
@@ -1268,8 +1295,8 @@ Tcl_KeylsetObjCmd (clientData, interp, objc, objv)
     }
 
     for (idx = 2; idx < objc; idx += 2) {
-        key = Tcl_GetStringFromObj (objv [idx], &keyLen);
-        if (ValidateKey (interp, key, keyLen, TRUE) == TCL_ERROR) {
+        key = Tcl_GetString(objv[idx]);
+        if (ValidateKey(interp, key, objv[idx]->length, TRUE) == TCL_ERROR) {
             goto errorExit;
         }
         if (TclX_KeyedListSet (interp, keylVarPtr, key, objv [idx+1]) != TCL_OK) {
@@ -1277,7 +1304,7 @@ Tcl_KeylsetObjCmd (clientData, interp, objc, objv)
         }
     }
 
-    if (Tcl_SetVar2Ex(interp, varName, NULL, keylVarPtr,
+    if (Tcl_ObjSetVar2(interp, objv[1], NULL, keylVarPtr,
                       TCL_LEAVE_ERR_MSG) == NULL) {
         goto errorExit;
     }
@@ -1305,25 +1332,24 @@ Tcl_KeyldelObjCmd (clientData, interp, objc, objv)
     Tcl_Obj     *const objv[];
 {
     Tcl_Obj *keylVarPtr, *keylPtr;
-    const char *varName, *key;
-    int idx, keyLen, status;
+    const char *key;
+    int idx, status;
 
     if (objc < 3) {
         return TclX_WrongArgs (interp, objv [0], "listvar key ?key ...?");
     }
-    varName = Tcl_GetStringFromObj (objv [1], NULL);
 
     /*
      * Get the variable that we are going to update.  If it is shared by more
      * than being a variable, duplicated it.
      */
-    keylVarPtr = Tcl_GetVar2Ex(interp, varName, NULL, TCL_LEAVE_ERR_MSG);
+    keylVarPtr = Tcl_ObjGetVar2(interp, objv[1], NULL, TCL_LEAVE_ERR_MSG);
     if (keylVarPtr == NULL) {
         return TCL_ERROR;
     }
     if (Tcl_IsShared (keylVarPtr)) {
         keylPtr = Tcl_DuplicateObj (keylVarPtr);
-        keylVarPtr = Tcl_SetVar2Ex(interp, varName, NULL, keylPtr, TCL_LEAVE_ERR_MSG);
+        keylVarPtr = Tcl_ObjSetVar2(interp, objv[1], NULL, keylPtr, TCL_LEAVE_ERR_MSG);
         if (keylVarPtr == NULL) {
             Tcl_DecrRefCount (keylPtr);
             return TCL_ERROR;
@@ -1335,8 +1361,8 @@ Tcl_KeyldelObjCmd (clientData, interp, objc, objv)
     keylPtr = keylVarPtr;
 
     for (idx = 2; idx < objc; idx++) {
-        key = Tcl_GetStringFromObj (objv [idx], &keyLen);
-        if (ValidateKey (interp, key, keyLen, TRUE) == TCL_ERROR) {
+        key = Tcl_GetString(objv[idx]);
+        if (ValidateKey(interp, key, objv[idx]->length, TRUE) == TCL_ERROR) {
             return TCL_ERROR;
         }
 
@@ -1368,15 +1394,14 @@ Tcl_KeylkeysObjCmd (clientData, interp, objc, objv)
     Tcl_Obj     *const objv[];
 {
     Tcl_Obj *keylPtr, *listObjPtr;
-    const char *varName, *key;
-    int keyLen, status;
+    const char *key;
+    int status;
 
     if ((objc < 2) || (objc > 3)) {
-        return TclX_WrongArgs (interp, objv [0], "listvar ?key?");
+        return TclX_WrongArgs(interp, objv [0], "listvar ?key?");
     }
-    varName = Tcl_GetStringFromObj (objv [1], NULL);
 
-    keylPtr = Tcl_GetVar2Ex(interp, varName, NULL, TCL_LEAVE_ERR_MSG);
+    keylPtr = Tcl_ObjGetVar2(interp, objv[1], NULL, TCL_LEAVE_ERR_MSG);
     if (keylPtr == NULL) {
         return TCL_ERROR;
     }
@@ -1388,8 +1413,8 @@ Tcl_KeylkeysObjCmd (clientData, interp, objc, objv)
     if (objc < 3) {
         key = NULL;
     } else {
-        key = Tcl_GetStringFromObj (objv [2], &keyLen);
-        if (ValidateKey (interp, key, keyLen, TRUE) == TCL_ERROR) {
+        key = Tcl_GetString(objv[2]);
+        if (ValidateKey(interp, key, objv[2]->length, TRUE) == TCL_ERROR) {
             return TCL_ERROR;
         }
     }
@@ -1421,10 +1446,13 @@ void
 TclX_KeyedListInit (interp)
     Tcl_Interp *interp;
 {
-    Tcl_RegisterObjType (&keyedListType);
+    Tcl_Obj *listobj;
+    Tcl_RegisterObjType(&keyedListType);
 
-    listType = Tcl_GetObjType("list");
-    stringType = Tcl_GetObjType("string");
+    listobj = Tcl_NewObj();
+    listobj = Tcl_NewListObj(1, &listobj);
+    listType = listobj->typePtr;
+    Tcl_DecrRefCount(listobj);
 
     if (0) {
     Tcl_CreateObjCommand (interp,
